@@ -1,39 +1,65 @@
 #' @name createFileRepositoryFolder
 #' @title Create a Folder in the File Repository
 #' 
-#' @description Creates a folder in the file repository. This can be made
-#'   a subfolder of an existing folder, given the folder ID of the parent
-#'   folder.
+#' @description This method enables the user to create a folder in the 
+#'   file repository. The folder created may also be a subfolder of an 
+#'   existing folder.
 #'   
-#' @param rcon A \code{redcapConnection} object.
-#' @param name \code{character(1)} The name of the folder. If a folder by 
+#' @inheritParams common-rcon-arg
+#' @inheritParams common-dot-args
+#' @inheritParams common-api-args 
+#' @param name `character(1)` The name of the folder. If a folder by 
 #'   this name already exists, no action will be taken.
-#' @param folder_id \code{integerish(0/1)}. The ID of the parent folder. If
+#' @param folder_id `integerish(0/1)`. The ID of the parent folder. When
 #'   length is 0, the new folder is placed in the top-level.
-#' @param dag_id \code{integerish(0/1)} The ID of a data access group. If 
+#' @param dag_id `integerish(0/1)` The ID of a data access group. When
 #'   provided, access to the folder will be restricted to the DAG.
-#' @param role_id \code{integerish(0/1)} The ID of a role. If provided, 
+#' @param role_id `integerish(0/1)` The ID of a role. When provided, 
 #'   access to the folder will be restricted to users with that role.
-#' @param ... Additional arguments to be passed between methods
-#' @param refresh \code{logical(1)} When \code{TRUE} (default), the cached 
-#'   File Repository data on \code{rcon} will be refreshed. 
-#' @param error_handling An option for how to handle errors returned by the API.
-#'   see \code{\link{redcap_error}}
-#' @param config \code{list} Additional configuration parameters to pass to 
-#'   \code{\link[httr]{POST}}. These are appended to any parameters in 
-#'   \code{rcon$config}.
-#' @param api_param \code{list} Additional API parameters to pass into the
-#'   body of the API call. This provides users to execute calls with options
-#'   that may not otherwise be supported by \code{redcapAPI}.
-#'   
-#' @details This method allows you to create a new folder in the File 
-#'   Repository. You may optionally provide the folder_id of the parent 
-#'   folder under which you wish this folder to be created. Providing a 
-#'   dag_id and/or role_id will allow you to restrict access to only users 
-#'   within a specific DAG (Data Access Group) or User Role, respectively.
-#'   
-#' @author Benjamin Nutter
+#' @param refresh `logical(1)` When `TRUE` (default), the cached 
+#'   File Repository data on `rcon` will be refreshed. 
+#'
+#' @return
+#' Returns a data frame with the columns
 #' 
+#' |             |                                                            |
+#' |-------------|------------------------------------------------------------|
+#' | `folder_id` | The REDCap assigned ID value for the newly created folder. |
+#' | `name`      | The name assigned to the folder by the user.               |
+#'
+#'   
+#' @seealso
+#' [exportFromFileRepository()], \cr
+#' [importToFileRepository()], \cr
+#' [deleteFromFileRepository()], \cr
+#' [exportFileRepository()], \cr
+#' [importFileRepository()], \cr
+#' [deleteFileRepository()], \cr
+#' [exportFileRepositoryListing()]
+#' 
+#' @examples
+#' \dontrun{
+#' unlockREDCap(connections = c(rcon = "project_alias"), 
+#'              url = "your_redcap_url", 
+#'              keyring = "API_KEYs", 
+#'              envir = globalenv())
+#'              
+#' # Create a folder in the top-level directory
+#' createFileRepositoryFolder(rcon, 
+#'                            name = "New Folder Name")
+#'                            
+#' # Create a folder in a subfolder
+#' createFileRepositoryFolder(rcon, 
+#'                            name = "New Folder Name", 
+#'                            folder_id = 12345)
+#'                            
+#' # Create a folder assigned to a Data Access Group
+#' createFileRepositoryFolder(rcon, 
+#'                            name = "New Folder Name", 
+#'                            dag_id = 678)
+#' }
+#' 
+#'   
 #' @export
 
 createFileRepositoryFolder <- function(rcon, 
@@ -151,7 +177,7 @@ createFileRepositoryFolder.redcapApiConnection <- function(rcon,
                           config)
   
   if (response$status_code != 200){
-    redcap_error(response, 
+    redcapError(response, 
                  error_handling = error_handling)
   } 
   
@@ -162,9 +188,9 @@ createFileRepositoryFolder.redcapApiConnection <- function(rcon,
   }
   
   # Prepare Output --------------------------------------------------
-  NewFolder <- read.csv(text = as.character(response), 
-                        stringsAsFactors = FALSE, 
-                        na.strings = "")
+  NewFolder <- utils::read.csv(text = as.character(response), 
+                               stringsAsFactors = FALSE, 
+                               na.strings = "")
   NewFolder$name <- rep(name, nrow(NewFolder))
   
   NewFolder
