@@ -1,207 +1,5 @@
-#' @name exportRecordsTyped
-#' 
-#' @title A replacement for \code{\link{exportRecords}} with full inversion of control over 
-#'        type casting.
-#' @description Exports records from a REDCap Database, allowing for 
-#'   subsets of subjects, fields, records, and events. This function is
-#'   the long term replacement for exportRecords. 
-#'
-#' @param rcon A REDCap connection object as created by \code{redcapConnection}.
-#' @param fields \code{character} vector of fields to be returned.  If \code{NULL}, 
-#'   all fields are returned (unless \code{forms} is specified).
-#' @param drop_fields \code{character} A vector of field names to remove from 
-#'   the export. Ignore if length = 0.
-#' @param forms \code{character} vector of forms to be returned.  If \code{NULL}, 
-#'   all forms are returned (unless \code{fields} is specified.
-#' @param records \code{character} or \code{integerish}. A vector of study id's 
-#'   to be returned.  If \code{NULL}, all subjects are returned.
-#' @param events A \code{character} vector of events to be returned from a 
-#'   longitudinal database.  If \code{NULL}, all events are returned. When 
-#'   using a \code{redcapOfflineConnection} object, this argument is unvalidated, 
-#'   and only rows that match one of the values given are returned; be advised
-#'   that misspellings may result in unexpected results.
-#' @param survey \code{logical(1)} specifies whether or not to export the survey identifier field 
-#'   (e.g., "redcap_survey_identifier") or survey timestamp fields 
-#'   (e.g., form_name+"_timestamp") when surveys are utilized in the project. 
-#'   If you do not pass in this flag, it will default to "true". If set to 
-#'   "true", it will return the redcap_survey_identifier field and also the 
-#'   survey timestamp field for a particular survey when at least 
-#'   one field from that survey is being exported. NOTE: If the survey 
-#'   identifier field or survey timestamp fields are imported via API data 
-#'   import, they will simply be ignored since they are not real fields in 
-#'   the project but rather are pseudo-fields.
-#' @param dag \code{logical(1)} specifies whether or not to export the "redcap_data_access_group" 
-#'   field when data access groups are utilized in the project. If you do not 
-#'   pass in this flag, it will default to "false". NOTE: This flag is only 
-#'   viable if the user whose token is being used to make the API request is 
-#'   *not* in a data access group. If the user is in a group, then this 
-#'   flag will revert to its default value.
-#' @param date_begin \code{POSIXct(1)}. Ignored if \code{NULL} (default). 
-#'   Otherwise, records created or modified after this date will be returned.
-#' @param date_end \code{POSIXct(1)}. Ignored if \code{NULL} (default). 
-#'   Otherwise, records created or modified before this date will be returned.
-#'
-#' @param na  A named \code{list} of user specified functions to determine if the
-#'   data is NA. This is useful when data is loaded that has coding for NA, e.g.
-#'   -5 is NA. Keys must correspond to a truncated REDCap field type, i.e.
-#'   {date_, datetime_, datetime_seconds_, time_mm_ss, time_hh_mm_ss, time, float,
-#'   number, calc, int, integer, select, radio, dropdown, yesno, truefalse,
-#'   checkbox, form_complete, sql}. The function will be provided the variables
-#'   (x, field_name, coding). The function must return a vector of logicals
-#'   matching the input. It defaults to \code{\link{isNAorBlank}} for all
-#'   entries.
-#' @param validation A named \code{list} of user specified validation functions. The 
-#'   same named keys are supported as the na argument. The function will be 
-#'   provided the variables (x, field_name, coding). The function must return a
-#'   vector of logical matching the input length. Helper functions to construct
-#'   these are \code{\link{valRx}} and \code{\link{valChoice}}. Only fields that
-#'   are not identified as NA will be passed to validation functions. 
-#' @param cast A named \code{list} of user specified class casting functions. The
-#'   same named keys are supported as the na argument. The function will be 
-#'   provided the variables (x, field_name, coding). The function must return a
-#'   vector of logical matching the input length. See \code{\link{fieldValidationAndCasting}}.
-#'   The field type \code{system} may also be used to determine how the fields
-#'   \code{redcap_event_name}, \code{redcap_repeat_instrument}, and 
-#'   \code{redcap_data_access_group} are cast.
-#' @param assignment A named \code{list} of functions. These functions are provided, field_name,
-#'   label, description and field_type and return a list of attributes to assign
-#'   to the column. Defaults to creating a label attribute from the stripped
-#'   HTML and UNICODE raw label and scanning for units={"UNITS"} in description
-#'   to use as a units attribute.
-#' @param config named \code{list}. Additional configuration parameters to pass to \code{httr::POST},
-#'   These are appended to any parameters in \code{rcon$config}
-#' @param api_param named \code{list}. Additional API parameters to pass into the body of the
-#'   API call. This provides users to execute calls with options that may not
-#'   otherwise be supported by redcapAPI.
-#' @param csv_delimiter character. One of \code{c(",", "\t", ";", "|", "^")}. Designates the
-#'   delimiter for the CSV file received from the API.
-#' @param batch_size \code{integerish(1)} (or \code{NULL}). If length \code{NULL},
-#'   all records are pulled. Otherwise, the records all pulled in batches of this size.
-#' @param ... Consumes any additional parameters passed. Not used.
-#' @param error_handling An option for how to handle errors returned by the API.
-#'   see \code{\link{redcap_error}}
-#'   
-#' @details
-#' 
-#' In all calls, the project's ID field will be included--there is no option
-#' provided to prevent this. Additionally, if the project has a secondary
-#' unique field specified, it will also be included. Inclusion of these fields
-#' is necessary to support some post-processing functions. 
-#' 
-#' By default, the system fields \code{redcap_event_name}, 
-#' \code{redcap_repeat_instrument}, and \code{redcap_repeat_instance} are 
-#' exported (when they are appropriate to the project). These are automatically
-#' included by the API. However, \code{exportRecordsTyped} assumes that if 
-#' you include only a subset of these system fields, then you truly only 
-#' want that subset and unrequested fields will be removed. Be aware that 
-#' this may cause problems with some post-processing functions that operate
-#' on repeating instrument data. 
-#' 
-#' The combination of the project ID field, secondary unique field, and the
-#' system fields are what uniquely identify an experimental unit. In nearly 
-#' all cases, it is desirable to have them all included.
-#' 
-#' A record of exports through the API is recorded in the Logging section 
-#' of the project.
-#' 
-#' The 'offline' version of the function operates on the raw (unlabeled) data 
-#' file downloaded from REDCap along with the data dictionary.  
-#' This is made available for instances where the API can not be accessed for 
-#' some reason (such as waiting for API approval from the REDCap administrator).
-#' 
-#' A 'batched' export is one where the export is performed over a series of 
-#' API calls rather than one large call.  For large projects on small servers, 
-#' this may prevent a single user from tying up the server and forcing others 
-#' to wait on a larger job.  The batched export is performed by first 
-#' calling the API to export the subject identifier field (the first field
-#' in the meta data).  The unique ID's are then assigned a batch number with 
-#' no more than \code{batch_size} ID's in any single batch.  The batches are 
-#' exported from the API and stacked together.
-#' 
-#' In longitudinal projects, \code{batch_size} may not necessarily be the 
-#' number of records exported in each batch.  If \code{batch_size} is 10 and 
-#' there are four records per patient, each batch will consist of 40 records.  
-#' Thus, if you are concerned about tying up the server with a large, 
-#' longitudinal project, it would be prudent to use a smaller batch size.
-#' 
-#' @section Inversion of Control:
-#' 
-#' The final product of calling this is a \code{data.frame} with columns
-#' that have been type cast to most commonly used analysis class (e.g. factor).
-#' This version allows the user to override any step of this process by
-#' specifying a different function for each of the stages of the type casting.
-#' The algorithm is as follows:
-#' 
-#' 1. Detect NAs in returned data (\code{na} argument).
-#' 2. Run \code{validate} functions for the field_types.
-#' 3. On the fields that are not NA and pass validate do the specified cast.
-#' 
-#' It is expected that the \code{na} and \code{validate} overrides should
-#' rarely be used. Their exposure via the function parameters is to future
-#' proof against possible bugs in the defaults, and allows for things that
-#' higher versions of REDCap add as possible field types. I.e., the overrides
-#' are for use to continue using the library when errors or changes to REDCap
-#' occur. 
-#' 
-#' The cast override is one were users can specify things that were controlled
-#' by an ever increasing set of flags before. E.g., \code{dates=as.Date} was
-#' an addition to allow dates in the previous version to be overridden if the 
-#' user wanted to use the Date class. In this version, it would appear called
-#' as \code{cast=list(_date=as.Date))}. See \code{\link{fieldValidationAndCasting}}
-#' for a full listing of package provided cast functions.
-#' 
-#' @section REDCap API Documentation (6.5.0):
-#' This function allows you to export a set of records for a project
-#' 
-#' Note about export rights (6.0.0+): Please be aware that Data Export user rights will be 
-#' applied to this API request. For example, if you have "No Access" data export rights 
-#' in the project, then the API data export will fail and return an error. And if you 
-#' have "De-Identified" or "Remove all tagged Identifier fields" data export rights, 
-#' then some data fields *might* be removed and filtered out of the data set returned 
-#' from the API. To make sure that no data is unnecessarily filtered out of your API 
-#' request, you should have "Full Data Set" export rights in the project.
-#' 
-#' @section REDCap Version:
-#' 5.8.2+ (Perhaps earlier) 
-#' 
-#' @section Known REDCap Limitations:
-#' None
-#' 
-#' @section Deidentified Batched Calls:
-#' Batched calls to the API are not a feature of the REDCap API, but may be imposed 
-#' by making multiple calls to the API.  The process of batching the export requires
-#' that an initial call be made to the API to retrieve only the record IDs.  The
-#' list of IDs is then broken into chunks, each about the size of \code{batch.size}.
-#' The batched calls then force the \code{records} argument in each call.
-#' 
-#' When a user's permissions require a de-identified data export, a batched call 
-#' should be expected to fail.  This is because, upon export, REDCap will hash the 
-#' identifiers.  When R attempts to pass the hashed identifiers back to REDCap, 
-#' REDCap will try to match the hashed identifiers to the unhashed identifiers in the
-#' database.  No matches will be found, and the export will fail.
-#' 
-#' Users who are exporting de-identified data will have to settle for using unbatched
-#' calls to the API (ie, \code{batch_size = -1})
-#' 
-#' @importFrom utils modifyList
-#' 
-#' @examples
-#' \dontrun{
-#' #Offline Connection example
-#' rcon_off <- 
-#'   offlineConnection(
-#'     meta_data = 
-#'       system.file(file.path("extdata/offlineConnectionFiles", 
-#'                             "TestRedcapAPI_DataDictionary.csv"), 
-#'                   package = "redcapAPI"), 
-#'     records = 
-#'       system.file(file.path("extdata/offlineConnectionFiles",
-#'                             "TestRedcapAPI_Records.csv"), 
-#'                   package = "redcapAPI"))
-#'
-#' exportRecordsTyped(rcon_off)
-#' }
-#' @seealso \code{\link{exportBulkRecords}}
+#' @describeIn recordsTypedMethods Export records with type casting.
+#' @order 1
 #' @export
 
 exportRecordsTyped <-
@@ -220,7 +18,8 @@ exportRecordsTyped <-
     
     UseMethod("exportRecordsTyped")
 
-#' @rdname exportRecordsTyped
+#' @rdname recordsTypedMethods
+#' @order 3
 #' @export
 
 exportRecordsTyped.redcapApiConnection <- 
@@ -245,6 +44,7 @@ exportRecordsTyped.redcapApiConnection <-
     cast           = list(),
     assignment     = list(label=stripHTMLandUnicode,
                           units=unitsFieldAnnotation),
+    filter_empty_rows = TRUE,
     ...,
     config         = list(),
     api_param      = list(),
@@ -314,6 +114,11 @@ exportRecordsTyped.redcapApiConnection <-
   checkmate::assert_list(x = api_param, 
                          names = "named", 
                          add = coll)
+  
+  checkmate::assert_logical(x = filter_empty_rows, 
+                            len = 1, 
+                            any.missing = FALSE,
+                            add = coll)
 
   checkmate::reportAssertions(coll)
   
@@ -330,7 +135,7 @@ exportRecordsTyped.redcapApiConnection <-
   user_requested_only_system_fields <- length(fields) > 0 && all(fields %in% REDCAP_SYSTEM_FIELDS)
   system_fields_user_requested <- REDCAP_SYSTEM_FIELDS[REDCAP_SYSTEM_FIELDS %in% fields]
   
-  # The REDCap API won't accept system fields in the fields argument. 
+  # The REDCap API will not accept system fields in the fields argument. 
   # we have to remove them from the request.
   fields <- fields[!fields %in% REDCAP_SYSTEM_FIELDS] # redcapDataStructures.R
   
@@ -359,7 +164,7 @@ exportRecordsTyped.redcapApiConnection <-
    ###################################################################
   # Call API for Raw Results
   
-  # We don't need to pass forms to the API because we have 
+  # We do not need to pass forms to the API because we have 
   # absorbed that information directly into fields
   body <- c(list(content                = "record", 
                  format                 = "csv", 
@@ -401,7 +206,7 @@ exportRecordsTyped.redcapApiConnection <-
     return(Raw)
   }
   
-  Raw <- filterEmptyRow(Raw, rcon)
+  if(filter_empty_rows) Raw <- filterEmptyRow(Raw, rcon)
   
   if (user_requested_system_fields){
     if (user_requested_only_system_fields){
@@ -426,8 +231,8 @@ exportRecordsTyped.redcapApiConnection <-
 
 
 
-# offline function ----------------------------------------------------
-#' @rdname exportRecordsTyped
+#' @describeIn recordsTypedMethods Export records without API access.
+#' @order 4
 #' @export
 
 exportRecordsTyped.redcapOfflineConnection <- function(rcon, 
@@ -494,7 +299,7 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
   user_requested_only_system_fields <- length(fields) > 0 && all(fields %in% REDCAP_SYSTEM_FIELDS)
   system_fields_user_requested <- REDCAP_SYSTEM_FIELDS[REDCAP_SYSTEM_FIELDS %in% fields]
   
-  # The REDCap API won't accept system fields in the fields argument. 
+  # The REDCap API will not accept system fields in the fields argument. 
   # we have to remove them from the request.
   fields <- fields[!fields %in% REDCAP_SYSTEM_FIELDS] # redcapDataStructures.R
   
@@ -720,7 +525,7 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
     rep((length(fields) == 0 && length(forms) == 0), 
         nrow(FieldFormMap))
   
-  # For the forms, we can't assume they are in forms. Instead, we initialize
+  # For the forms, we cannot assume they are in forms. Instead, we initialize
   # this to FALSE and have to provide positive proof that they are in forms.
   FieldFormMap$is_in_forms <- rep(FALSE, nrow(FieldFormMap))
   
@@ -753,7 +558,7 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
   
   # If any of the checkbox options are listed if drop_fields
   # Remove all of the checkbox options.
-  # Also sets the is_in_forms to FALSE to ensure it isn't 
+  # Also sets the is_in_forms to FALSE to ensure it is not 
   # included in the API call.
   
   if (length(drop_fields) > 0){
@@ -814,7 +619,7 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
                           config = config)
   
   if (response$status_code != 200){
-    redcap_error(response, 
+    redcapError(response, 
                  error_handling = error_handling)
   } 
   
@@ -823,11 +628,11 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
     return(data.frame())
   }
   
-  read.csv(text = as.character(response), 
-           stringsAsFactors = FALSE, 
-           na.strings = "", 
-           colClasses = "character", 
-           sep = csv_delimiter)
+  utils::read.csv(text = as.character(response), 
+                  stringsAsFactors = FALSE, 
+                  na.strings = "", 
+                  colClasses = "character", 
+                  sep = csv_delimiter)
 }
 
 # .exportRecordsTyped_Batched ---------------------------------------
@@ -852,7 +657,7 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
                                                                 "fields")))
     
     if (record_response$status_code != 200){
-      redcap_error(record_response, 
+      redcapError(record_response, 
                    error_handling = error_handling)
     }
     
@@ -861,10 +666,10 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
       return(data.frame())
     }
     
-    records <- read.csv(text = as.character(record_response), 
-                        stringsAsFactors = FALSE, 
-                        na.strings = "", 
-                        sep = csv_delimiter)
+    records <- utils::read.csv(text = as.character(record_response), 
+                               stringsAsFactors = FALSE, 
+                               na.strings = "", 
+                               sep = csv_delimiter)
     records <- records[[target_field]]
   }
   
@@ -899,25 +704,25 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
 #' @title A helper function to export multiple records and forms using
 #' a single call.
 #' @description Exports records from multiple REDCap Databases using
-#' multiple calls to \code{\link{exportRecordsTyped}}
+#' multiple calls to [exportRecordsTyped()]
 #'
-#' @param rcon A named list of REDCap connection object as created by \code{\link{redcapConnection}}.
-#' @param forms A named list that is a subset of rcon's names. A specified \code{rcon}
-#'              will provide a list of forms for repeated calls to \code{exportRecordsType}.
+#' @inheritParams common-rcon-arg
+#' @param forms A named list that is a subset of rcon's names. A specified `rcon`
+#'              will provide a list of forms for repeated calls to `exportRecordsType`.
 #'              If a connection reference is missing it will default to all forms. To override
 #'              this default specify a connection's forms with NA to just get all
 #'              data. 
 #' @param envir A environment to write the resulting Records in as variables
 #'   given by their name in rcon or if from a form their rcon named pasted to 
-#'   their form name joined by \code{sep}. If not specified the function
+#'   their form name joined by `sep`. If not specified the function
 #'   will return a named list with the results. Will accept a number of the
 #'   environment.
 #' @param sep A character string to use when joining the rcon name to the form name
 #' for storing variables. 
 #' @param post A function that will run on all returned sets of Records. 
-#' @param \dots Any additional variables to pass to \code{\link{exportRecordsTyped}}.
-#' @return Will return a named list of the resulting records if \code{envir} is 
-#'    NULL. Otherwise will assign them to the specified \code{envir}.
+#' @param \dots Any additional variables to pass to [exportRecordsTyped()].
+#' @return Will return a named list of the resulting records if `envir` is 
+#'    NULL. Otherwise will assign them to the specified `envir`.
 #' @examples
 #' \dontrun{
 #' unlockREDCap(c(test_conn    = 'TestRedcapAPI',
@@ -952,6 +757,7 @@ exportRecordsTyped.redcapOfflineConnection <- function(rcon,
 #'# post processed in the same manner as specified by `post`.
 #' }
 #' @export
+
 exportBulkRecords <- function(rcon, forms=NULL, envir=NULL, sep="_", post=NULL, ...)
 {
   if(is.numeric(envir)) envir <- as.environment(envir)
@@ -1031,4 +837,4 @@ exportBulkRecords <- function(rcon, forms=NULL, envir=NULL, sep="_", post=NULL,
   }
   
   if(is.null(envir)) dest else list2env(dest, envir=envir)
-}
\ No newline at end of file
+}
