@@ -1,102 +1,173 @@
 #' @name fieldValidationAndCasting
-#' @title Helper functions for \code{exportRecordsTyped} Validation and Casting
+#' @title Helper functions for `exportRecordsTyped` Validation and Casting
 #' @description This set of functions assists in validating that the content of 
 #'   fields coming from REDCap match the MetaData, allowing for a 
 #'   validation report to provided. The cast helpers allow for transforming
 #'   the REDCap data into R data types and allowing the user to customize 
 #'   the end product.
 #'   
-#' @param x \code{character}. A vector to check.
-#' @param rx \code{character}. The regular expression pattern to check.
-#' @param coding named \code{character} vector. The defined coding from the meta data.
-#' @param field_name \code{character(1)}. Name of the field(s)
-#' @param dec_symbol \code{character(1)}. The symbol in the field used to 
+#' @param x `character`. A vector to check.
+#' @param rx `character`. The regular expression pattern to check.
+#' @param coding named `character` vector. The defined coding from the meta data.
+#' @param field_name `character(1)`. Name of the field(s)
+#' @param dec_symbol `character(1)`. The symbol in the field used to 
 #'   denote a decimal. 
-#' @param n_dec \code{integerish(1)}. The number of decimal places permitted
+#' @param n_dec `integerish(1)`. The number of decimal places permitted
 #'   by the field validation. 
-#' @param checked \code{character}. Values to recognize as checked in a 
+#' @param checked `character`. Values to recognize as checked in a 
 #'   checkbox field.
-#' @param FUN \code{function}. A function that takes a character vector. 
+#' @param FUN `function`. A function that takes a character vector. 
 #' @param ... Consumes anything else passed to function. I.e., field_name and 
 #' coding.
 #'
-#' @details Functions passed to the \code{na}, \code{validation}, and
-#' \code{cast} parameter of \code{\link{exportRecordsTyped}} all take the form
-#' of \code{function(x, coding, field_name)}. \code{na} and \code{validation}
+#' @details Functions passed to the `na`, `validation`, and
+#' `cast` parameter of [exportRecordsTyped()] all take the form
+#' of `function(x, coding, field_name)`. `na` and `validation`
 #' functions are expected to return a logical vector of the same length as the
 #' column processed. Helper routines
 #' are provided here for common cases to construct these functions. 
 #' 
-#' \code{isNAorBlank} returns TRUE/FALSE if field is NA or blank. Helper
-#' function for constructing \code{na} overrides in \code{\link{exportRecordsTyped}}.
+#' ## Validation Functions
 #' 
-#' \code{valRx} constructs a validation function from a regular expression pattern. 
+#' `isNAorBlank` returns TRUE/FALSE if field is NA or blank. Helper
+#' function for constructing `na` overrides in [exportRecordsTyped()].
+#' 
+#' `valRx` constructs a validation function from a regular expression pattern. 
 #' The function returns a TRUE/FALSE if the value matches the pattern.
 #' 
-#' \code{valChoice} constructs a validation function from a set of choices 
+#' `valChoice` constructs a validation function from a set of choices 
 #' defined in the MetaData. The functions returns a TRUE/FALSE if the value
 #' matches one of the choices.
 #' 
-#' \code{valPhone} constructs a validation function for (North American) 
+#' `valPhone` constructs a validation function for (North American) 
 #'   phone numbers. It removes punctuation and spaces prior to validating
 #'   with the regular expression.
+#'   
+#' `na_values` is a helper function to create a list of functions
+#' to test for NA based on field type. Useful for bulk override of
+#' NA detection for a project. The output can be directly passed to the `na`
+#' parameter of [exportRecordsTyped()].
 #' 
-#' \code{castLabel} constructs a casting function for multiple choice variables. 
-#' The field will be cast to return the choice label (generally more human readable)
+#' ## Casting Functions
 #' 
-#' \code{castCode} constructs a casting function for multiple choice variables.
-#' Similar to \code{castLabel}, but the choice value is returned instead. The
+#' `castLabel` constructs a casting function for multiple choice variables. 
+#' The field will be cast to return the choice label (generally more human readable). 
+#' `castLabelCharacter` is an equivalent casting function that returns 
+#' a `character` vector instead of a `factor`.
+#' 
+#' `castCode` constructs a casting function for multiple choice variables.
+#' Similar to `castLabel`, but the choice value is returned instead. The
 #' values are typically more compact and their meaning may not be obvious.
+#' `castCodeCharacter` is an equivalent casting function that retuns
+#' a `character` vector instead of a `factor`.
 #' 
-#' \code{castRaw} constructs a casting function that returns the content
-#' from REDCap as it was received. It is functionally equivalent to \code{identity}. 
+#' `castRaw` constructs a casting function that returns the content
+#' from REDCap as it was received. It is functionally equivalent to `identity`. 
 #' For multiple choice variables, the result will be coerced to numeric, if possible; 
-#' otherwise, the result is character vector.
+#' otherwise, the result is character vector. 
 #' 
-#' \code{castChecked} constructs a casting function for checkbox fields. It
-#' returns values in the form of Unchecked/Checked.
+#' `castChecked` constructs a casting function for checkbox fields. It
+#' returns values in the form of Unchecked/Checked. `castCheckedCharacter`
+#' is an equivalent casting function that returns a `character` vector 
+#' instead of a `factor`.
 #' 
-#' \code{castCheckLabel} and \code{castCheckCode} also construct casting functions
+#' `castCheckLabel` and `castCheckCode` also construct casting functions
 #' for checkbox fields. For both, unchecked variables are cast to an empty 
 #' string (""). Checked variables are cast to the option label and option code, 
-#' respectively.
+#' respectively. `castCheckLabelCharacter` and `castCheckCodeCharacter`
+#' are equivalent casting functions that returns a `character` vector 
+#' instead of a `factor`.
 #' 
-#' \code{castCheckForImport} is a special case function to allow the user to
+#' `castCheckForImport` is a special case function to allow the user to
 #' specify exactly which values are to be considered "Checked". Values that
-#' match are returned as 1 and all other values are returned as 0. This is
+#' match are returned as `1` and all other values are returned as `0`. This is
 #' motivated by the special case where the coding on a checkbox includes 
-#' "0, Option". In the resulting field \code{checkbox___0}, a coded value
-#' of 0 actually implies the choice was selected. In order to perform an 
+#' "0, Option". In the resulting field `checkbox___0`, a coded value
+#' of `0` actually implies the choice was selected. In order to perform an 
 #' import on such data, it is necessary to cast it using 
-#' \code{castCheckForImport(c("0"))}.
+#' `castCheckForImport(c("0"))`.
 #' 
-#' \code{castDpNumeric} is a casting function for fields that use the 
-#' \code{number_ndp_comma} field type (where \code{n} is the number of 
+#' `castDpNumeric` is a casting function for fields that use the 
+#' `number_ndp_comma` field type (where `n` is the number of 
 #' decimal places). This function will convert the values to numeric
 #' values for use in analysis. This is a function that returns the 
 #' appropriate casting function, thus the appropriate usage when using 
-#' the defaults is \code{cast = list(number_1dp_comma = castDpNumeric())}
+#' the defaults is `cast = list(number_1dp_comma = castDpNumeric())`
 #' (using the parentheses).
 #' 
-#' \code{castDpCharacter} is a casting function to return fields that use
-#' \code{number_ndp_comma} field types to character strings for import. This 
+#' `castDpCharacter` is a casting function to return fields that use
+#' `number_ndp_comma` field types to character strings for import. This 
 #' is a function that returns the appropriate casting function, thus the 
 #' appropriate usage when casting for one decimal place is 
-#' \code{cast = list(number_1dp_comma = castDpCharacter(1))}.
+#' `cast = list(number_1dp_comma = castDpCharacter(1))`.
 #' 
-#' \code{castTimeHHMM} and \code{castTimeMMSS} are casting functions to 
+#' `castTimeHHMM` and `castTimeMMSS` are casting functions to 
 #' facilitate importing data. They convert time data into a character format 
 #' that will pass the API requirements. 
 #' 
-#' \code{raw_cast} overrides all casting if passed as the \code{cast}
+#' 
+#' ## Casting Lists
+#' `raw_cast` overrides all casting if passed as the `cast`
 #' parameter.
 #' 
-#' \code{na_values} A helper function to create a list of functions
-#' to test for NA based on field type. Useful for bulk override of
-#' NA detection for a project. The output can be directly passed to the \code{na}
-#' parameter of \code{\link{exportRecordsTyped}}.
+#' `default_cast_no_factor` is a list of casting functions that matches
+#' all of the default casts but with the exception that any fields that would
+#' have been cast to factors will instead be cast to characters. It is 
+#' provided for the user that prefers to work absent factors. The list
+#' `default_cast_character` is equivalent and is provided for those that
+#' prefer to describe their casting in terms of what the result is (and not
+#' what it is not).
+#' 
+#' 
+#' @return 
+#' 
+#' Validation and casting functions return the objects indicated in the 
+#' following table:
+#' 
+#' | Function Name             | Object Type Returned |
+#' |---------------------------|----------------------| 
+#' | `isNAOrBlank`             | `logical`            |
+#' | `valRx`                   | `logical`            |
+#' | `valChoice`               | `logical`            |
+#' | `valPhone`                | `logical`            |
+#' | `castLabel`               | `factor`             |
+#' | `castLabelCharacter`      | `character`          |
+#' | `castCode`                | `factor`             |
+#' | `castCodeCharacter`       | `character`          |
+#' | `castRaw`                 | `character`          |
+#' | `castChecked`             | `factor`             |
+#' | `castCheckedCharacter`    | `character`          |
+#' | `castCheckLabel`          | `factor`             |
+#' | `castCheckLabelCharacter` | `character`          |
+#' | `castCheckCode`           | `factor`             |
+#' | `castCheckCodeCharacter`  | `character`          | 
+#' | `castCheckForImport`      | `numeric`            |
+#' | `castDpNumeric`           | `numeric`            |
+#' | `castDpCharacter`         | `character`          |
+#' | `castTimeHHMM`            | `character`          |
+#' | `castTimeMMSS`            | `character`          |
+#' 
+#' 
+#' @seealso 
+#' [fieldCastingFunctions()], \cr
+#' [exportRecordsTyped()], \cr
+#' [exportReportsTyped()]
+#' 
+#' @examples
+#' \dontrun{
+#' # Make a custom function to give special treatment to a field.
+#' # In this function, the field "field_name_to_skip" will 
+#' # be cast using `castRaw`. All other fields will be cast 
+#' # using `castCode`
+#' customCastCode <- function(x, field_name, coding){
+#'   if (field_name == "field_name_to_skip"){
+#'     castRaw(x, field_name, coding)
+#'   } else {
+#'     castCode(x, field_name, coding)
+#'   }
+#' }
+#' }
 #' 
-#' @author Shawn Garbett, Benjamin Nutter
 
 #####################################################################
 # Validation                                                     ####
@@ -121,6 +192,21 @@ valPhone <- function(x, field_name, coding){
   grepl(REGEX_PHONE, x)
 }
 
+#' @rdname fieldValidationAndCasting
+#' @export
+na_values <- function(FUN)
+{
+  coll <- checkmate::makeAssertCollection()
+  checkmate::assert_function( x       = FUN,
+                              null.ok = FALSE,
+                              add     = coll)
+  checkmate::reportAssertions(coll)
+  
+  l <- lapply(FIELD_TYPES, function(f) FUN)
+  names(l) <- FIELD_TYPES
+  l
+}
+
 #####################################################################
 # Casting                                                        ####
 
@@ -132,6 +218,12 @@ castLabel <- function(x, field_name, coding){
   factor(unname(coding[code_match]), levels = coding, labels = names(coding))
 }
 
+#' @rdname fieldValidationAndCasting
+#' @export
+castLabelCharacter <- function(x, field_name, coding){
+  as.character(castLabel(x, field_name, coding))
+}
+
 #' @rdname fieldValidationAndCasting
 #' @export
 castCode <- function(x, field_name, coding){
@@ -140,12 +232,22 @@ castCode <- function(x, field_name, coding){
   factor(unname(coding[code_match]), levels = coding, labels = coding)
 }
 
+#' @rdname fieldValidationAndCasting
+#' @export
+castCodeCharacter <- function(x, field_name, coding){
+  as.character(castCode(x, field_name, coding))
+}
+
 #' @rdname fieldValidationAndCasting
 #' @export
 castRaw <- function(x, field_name, coding){
+  warnOfZeroCodedCheckCasting(field_name, x)
+  
   raw <- 
     if (grepl(".*___(.*)", field_name)){
-      as.character((x %in% getCheckedValue(coding, field_name)) + 0L)
+      ifelse(!is.na(x), 
+             as.character((x %in% getCheckedValue(coding, field_name)) + 0L), 
+             NA)
     } else {
       code_match <- getCodingIndex(x, coding)
       unname(coding[code_match])
@@ -158,21 +260,45 @@ castRaw <- function(x, field_name, coding){
 castChecked <- function(x, field_name, coding){
   checked_value <- getCheckedValue(coding, field_name)
   
+  warnOfZeroCodedCheckCasting(field_name, x)
+  
   x_checked <- x %in% checked_value 
   
   factor(c("Unchecked", "Checked")[(x_checked)+1], levels=c("Unchecked", "Checked"))
 }
 
+#' @rdname fieldValidationAndCasting
+#' @export
+castCheckedCharacter <- function(x, field_name, coding){
+  as.character(castChecked(x, field_name, coding))
+}
+
 #' @rdname fieldValidationAndCasting
 #' @export
 castCheckLabel <- function(x, field_name, coding){
   checked_value <- getCheckedValue(coding, field_name)
-
+  
+  warnOfZeroCodedCheckCasting(field_name, x)
+  
   x_checked <- x %in% checked_value 
   
-  factor(unname(c("", checked_value[1])[(x_checked) + 1]), 
-         levels=c("", checked_value[1]), 
-         labels=c("", names(checked_value)[1]))
+  # Sets the level and label while accomodating 0 coded check values
+  # (0 is not considered a `checked_value` in this case, so must be handled by force)
+  
+  is_zero_coded <- isZeroCodedCheckField(field_name)
+  
+  the_level <- if (is_zero_coded) "0"              else checked_value[1]
+  the_label <- if (is_zero_coded) checked_value[1] else names(checked_value)[1]
+  
+  factor(unname(c("", the_level)[(x_checked) + 1]), 
+         levels=c("", the_level), 
+         labels=c("", the_label))
+}
+
+#' @rdname fieldValidationAndCasting
+#' @export
+castCheckLabelCharacter <- function(x, field_name, coding){
+  as.character(castCheckLabel(x, field_name, coding))
 }
 
 #' @rdname fieldValidationAndCasting
@@ -180,11 +306,27 @@ castCheckLabel <- function(x, field_name, coding){
 castCheckCode <- function(x, field_name, coding){
   checked_value <- getCheckedValue(coding, field_name)
   
+  warnOfZeroCodedCheckCasting(field_name, x)
+  
   x_checked <- x %in% checked_value 
   
-  factor(unname(c("", checked_value[1])[(x_checked) + 1]), 
-         levels=c("", checked_value[1]), 
-         labels=c("", checked_value[1]))
+  # Sets the level and label while accomodating 0 coded check values
+  # (0 is not considered a `checked_value` in this case, so must be handled by force)
+  
+  is_zero_coded <- isZeroCodedCheckField(field_name)
+  
+  the_level <- if (is_zero_coded) "0" else checked_value[1]
+  the_label <- if (is_zero_coded) "0" else checked_value[1]
+  
+  factor(unname(c("", the_level)[(x_checked) + 1]), 
+         levels=c("", the_level), 
+         labels=c("", the_label))
+}
+
+#' @rdname fieldValidationAndCasting
+#' @export
+castCheckCodeCharacter <- function(x, field_name, coding){
+  as.character(castCheckCode(x, field_name, coding))
 }
 
 #' @rdname fieldValidationAndCasting
@@ -192,13 +334,17 @@ castCheckCode <- function(x, field_name, coding){
 
 castCheckForImport <- function(checked = c("Checked", "1")){
   function(x, coding, field_name){
-    (x %in% checked) + 0L
+    is_na <- is.na(x)
+    
+    out <- (x %in% checked) + 0L
+    out[is_na] <- NA
+    out
   }
 }
 
 # utility function returns the index of the codebook matching the 
 # content of the vector. Permits accurate matching without foreknowledge
-# of whether the data are coded or labelled.
+# of whether the data are coded or labeled.
 getCodingIndex <- function(x, coding){
   code_match <- match(as.character(x), coding)
   
@@ -216,12 +362,19 @@ getCheckedValue <- function(coding, field_name){
   # for instance, a checkbox with code 4-3, label produce variable name checkbox___4_3 
   this_code_index <- match(this_code, 
                            tolower(gsub("[[:punct:]]", "_", coding)))  
-
+  
   checked_value <- c(coding[this_code_index], 
                      names(coding)[this_code_index], 
                      "1", 
                      "Checked", 
                      "yes")
+  
+  # When casting from raw, we do not want to consider "0" as checked 
+  # for zero coded fields.
+  if (isZeroCodedCheckField(field_name)){
+    checked_value <- checked_value[-1]
+  } 
+  
   checked_value
 }
 
@@ -312,8 +465,44 @@ raw_cast <- list(
   system             = NA
 )
 
+#' @rdname fieldValidationAndCasting
+#' @export
+
+default_cast_no_factor <- list(
+  date_                    = function(x, ...) as.POSIXct(x, format = "%Y-%m-%d"),
+  datetime_                = function(x, ...) as.POSIXct(x, format = "%Y-%m-%d %H:%M"),
+  datetime_seconds_        = function(x, ...) as.POSIXct(x, format = "%Y-%m-%d %H:%M:%S"),
+  time_mm_ss               = function(x, ...) chron::times(ifelse(is.na(x),NA,paste0("00:",x)), format=c(times="h:m:s")),
+  time_hh_mm_ss            = function(x, ...) chron::times(x, format=c(times="h:m:s")),
+  time                     = function(x, ...) chron::times(gsub("(^\\d{2}:\\d{2}$)", "\\1:00", x), 
+                                                           format=c(times="h:m:s")),
+  float                    = as.numeric,
+  number                   = as.numeric,
+  number_1dp               = as.numeric, 
+  number_1dp_comma_decimal = castDpNumeric(),
+  number_2dp               = as.numeric, 
+  number_2dp_comma_decimal = castDpNumeric(),
+  calc                     = as.numeric,
+  int                      = as.integer,
+  integer                  = as.numeric,
+  yesno                    = castLabelCharacter,
+  truefalse                = function(x, ...) x=='1' | tolower(x) =='true',
+  checkbox                 = castCheckedCharacter,
+  form_complete            = castLabelCharacter,
+  select                   = castLabelCharacter,
+  radio                    = castLabelCharacter,
+  dropdown                 = castLabelCharacter,
+  sql                      = NA, 
+  system                   = castLabelCharacter
+)
+
+#' @rdname fieldValidationAndCasting
+#' @export
+
+default_cast_character <- default_cast_no_factor
+
 #####################################################################
-# Unexported - default lists for exportRecordsTyped
+# Unexported - default lists for exportRecordsTyped              ####
 
 .default_validate <- list(
   date_              = valRx(REGEX_DATE),              # REGEX values defined in constants.R
@@ -431,6 +620,9 @@ raw_cast <- list(
   system                   = castRaw
 )
 
+#####################################################################
+# FIELD_TYPES constant                                           ####
+
 FIELD_TYPES <- c(
   "date_",          "datetime_",  "datetime_seconds_",  "time_mm_ss",
   "time_hh_mm_ss",  "time",       "float",              "number",
@@ -438,18 +630,5 @@ FIELD_TYPES <- c(
   "truefalse",      "checkbox",   "form_complete",      "select",
   "radio",          "dropdown",   "sql")
 
-#' @rdname fieldValidationAndCasting
-#' @export
-na_values <- function(FUN)
-{
-  coll <- checkmate::makeAssertCollection()
-  checkmate::assert_function( x       = FUN,
-                              null.ok = FALSE,
-                              add     = coll)
-  checkmate::reportAssertions(coll)
-  
-  l <- lapply(FIELD_TYPES, function(f) FUN)
-  names(l) <- FIELD_TYPES
-  l
-}
+
 
