@@ -1,103 +1,106 @@
 #' @name importRecords
-#' @title Import Records to a REDCap Database
-#' 
-#' @description Imports records from a \code{data.frame} to a REDCap Database
+#' @title Import Records to a Project
 #'
-#' @param rcon A REDCap connection object as created by \code{redcapConnection}.
-#' @param data A \code{data.frame} to be imported to the REDCap project.
-#' @param overwriteBehavior Character string.  'normal' prevents blank
-#'   fields from overwriting populated fields.  'overwrite' causes blanks to
-#'   overwrite data in the REDCap database.
-#' @param returnContent Character string.  'count' returns the number of
-#'   records imported; 'ids' returns the record ids that are imported;
-#'   'nothing' returns no message; 'auto_ids' returns a list of pairs of all 
-#'   record IDs that were imported. If used when \code{force_auto_number = FALSE}, 
-#'   the value will be changed to \code{'ids'}.
-#' @param returnData Logical.  Prevents the REDCap import and instead
-#'   returns the data frame that would have been given
+#' @description These methods enable the user to import new records or update
+#'   existing records to a project. 
+#'   
+#' @inheritParams common-rcon-arg
+#' @inheritParams common-dot-args
+#' @inheritParams common-api-args
+#' @param data A `data.frame` to be imported to the project.
+#' @param overwriteBehavior `character(1)`. One of `c("normal", "overwrite")`. 
+#'   `"normal"` prevents blank fields from overwriting populated fields.  
+#'   `"overwrite"` causes blanks to overwrite data in the database.
+#' @param force_auto_number `logical(1)`. If record auto-numbering has been
+#'   enabled in the project, it may be desirable to import records where each 
+#'   record's record name is automatically determined by REDCap (just as it 
+#'   does in the user interface). When `TRUE`, the 
+#'   record names provided in the request will not be used (although they 
+#'   are still required in order to associate multiple rows of data to an 
+#'   individual record in the request); instead those records in the 
+#'   request will receive new record names during the import process. 
+#'   It is recommended that the user use `returnContent = "auto_ids"`
+#'   when `force_auto_number = TRUE`
+#' @param returnContent `character(1)`.  
+#'   One of `c("count", "ids", "nothing", "auto_ids")`.
+#'   'count' returns the number of records imported; 
+#'   'ids' returns the record ids that are imported;
+#'   'nothing' returns no message; 
+#'   'auto_ids' returns a list of pairs of all record IDs that were imported. 
+#'   If used when `force_auto_number = FALSE`, the value will be changed to `'ids'`.
+#' @param returnData `logical(1)`. When `TRUE`, prevents the REDCap 
+#'   import and instead returns the data frame that would have been given
 #'   for import. This is sometimes helpful if the API import fails without
 #'   providing an informative message. The data frame can be written to a csv
 #'   and uploaded using the interactive tools to troubleshoot the
-#'   problem. Please send us an e-mail if you find errors I haven't
-#'   accounted for.
-#' @param logfile An optional filepath (preferably .txt) in which to print the
-#'   log of errors and warnings about the data.
-#'   If \code{""}, the log is printed to the console.
-#' @param force_auto_number \code{logical(1)} If record auto-numbering has been
-#'   enabled in the project, it may be desirable to import records where each 
-#'   record's record name is automatically determined by REDCap (just as it 
-#'   does in the user interface). If this parameter is set to 'true', the 
-#'   record names provided in the request will not be used (although they 
-#'   are still required in order to associate multiple rows of data to an 
-#'   individual record in the request), but instead those records in the 
-#'   request will receive new record names during the import process. 
-#'   NOTE: To see how the provided record names get translated into new auto
-#'    record names, the returnContent parameter should be set to 'auto_ids', 
-#'    which will return a record list similar to 'ids' value, but it will have
-#'    the new record name followed by the provided record name in the request, 
-#'    in which the two are comma-delimited.
-#' @param batch.size Specifies size of batches.  A negative value
-#'   indicates no batching.
-#' @param ... Arguments to be passed to other methods.
-#' @param error_handling An option for how to handle errors returned by the API.
-#'   see \code{\link{redcap_error}}
-#' @param config \code{list} Additional configuration parameters to pass to 
-#'   \code{\link[httr]{POST}}. These are appended to any parameters in 
-#'   \code{rcon$config}.
-#' @param api_param \code{list} Additional API parameters to pass into the
-#'   body of the API call. This provides users to execute calls with options
-#'   that may not otherwise be supported by \code{redcapAPI}.
+#'   problem. 
+#' @param logfile `character(1)`. An optional filepath (preferably .txt) 
+#'   in which to print the log of errors and warnings about the data.
+#'   When `""`, the log is printed to the console. 
+#' @param batch.size `integerish(1)`.  Specifies the number of subjects to be included 
+#'   in each batch of a batched export or import.  Non-positive numbers 
+#'   export/import the entire operation in a single batch. 
+#'   Batching may be beneficial to prevent tying up smaller servers.  
+#'   See Details.
 #'
 #' @details
-#' A record of imports through the API is recorded in the Logging section
-#' of the project.
-#'
-#' \code{importRecords} prevents the most common import errors by testing the
+#' `importRecords` prevents the most common import errors by testing the
 #' data before attempting the import.  Namely
-#' \enumerate{
-#'   \item Check that all variables in \code{data} exist in the REDCap data dictionary.
-#'   \item Check that the study id variable exists
-#'   \item Force the study id variable to the first position in the data frame (with a warning)
-#'   \item Remove calculated fields (with a warning)
-#'   \item Verify that REDCap date fields are represented in the data frame as
-#'     either character, POSIXct, or Date class objects.
-#'   \item Determine if values are within their specified validation limits.
-#' }
-#'
-#' See the documentation for \code{\link{validateImport}} for detailed
-#' explanations of the validation.
 #' 
-#' @section Limitations:
+#' 1. Check that all variables in `data` exist in the REDCap data dictionary.
+#' 2. Check that the record id variable exists
+#' 3. Force the record id variable to the first position in the data frame (with a warning)
+#' 4. Remove calculated fields (with a warning)
+#' 5. Verify that REDCap date fields are represented in the data frame as either `character`, `POSIXct`, or `Date` class objects.
+#' 6. Determine if values are within their specified validation limits.
+#'
+#' See the documentation for [validateImport()] for detailed
+#' explanations of the validation. 
+#'  
+#' A 'batched' import is one where the export is performed over a series of 
+#' API calls rather than one large call.  For large projects on small servers, 
+#' this may prevent a single user from tying up the server and forcing others 
+#' to wait on a larger job.  
 #' 
-#' The REDCap API is fairly restrictive about what it will accept as valid data for import. 
-#' \code{importRecords} tries to simplify the process by allowing users to 
-#' submit data in any form recognized by the data dictionary. It is then converted
-#' internally to the 
-#' appropriate text format for import. This means, for example, that a radio button value
-#' where the code \code{1} is mapped to the label \code{Guitar} (defined in the user interface
-#' with "1, Guitar"), the user can provide
-#' either "1" or "Guitar" as a value and \code{importRecords} will translate it to the 
-#' code that the API expects. 
+#' @return
+#' `importRecords`, when `returnData = FALSE`, returns the content from the
+#'   API response designated by the `returnContent` argument. 
+#'   
+#' `importRecords`, when `returnData = TRUE`, returns the 
+#'   data frame that was internally prepared for import. This data frame has
+#'   values transformed from R objects to character values the API will 
+#'   accept. 
+#'
+#' @seealso 
+#' [exportRecords()], \cr
+#' [deleteRecords()], \cr
+#' [exportRecordsTyped()]
 #' 
-#' While this provides a level of convenience for the user, it has some limitations when
-#' applied to checkbox values. When submitting checkbox values for import, it is strongly 
-#' recommended that you submit either the code "0" (for unchecked), "1" (for checked), or the 
-#' labels "Unchecked" and "Checked". 
+#' @examples
+#' \dontrun{
+#' unlockREDCap(connections = c(rcon = "project_alias"), 
+#'              url = "your_redcap_url", 
+#'              keyring = "API_KEYs", 
+#'              envir = globalenv())
+#' 
+#' # Import records
+#' NewData <- data.frame(record_id = c(1, 2, 3), 
+#'                       age = c(27, 43, 32), 
+#'                       date_of_visit = rep(Sys.Date(), 3))
+#' importRecords(rcon, 
+#'               data = NewData)
+#'               
+#'               
+#' # Import records and save validation info to a file
+#' NewData <- data.frame(record_id = c(1, 2, 3), 
+#'                       age = c(27, 43, 32), 
+#'                       date_of_visit = rep(Sys.Date(), 3))
+#' importRecords(rcon, 
+#'               data = NewData, 
+#'               logfile = "import-validation-notes.txt")      
+#' 
+#' } 
 #' 
-#' In particular, when the checkbox labels are defined with a code or label that is "0" or "1"
-#' (for example, "0, checkbox_label" or "check_code, 0"), \code{importRecords} is unable to 
-#' determine if a 0 indicates an unchecked box or if the zero is the label of a checked box. 
-#' When encountering ambiguity, \code{importRecords} will always assume "0" represents an
-#' unchecked box and "1" represents a checked box.
-#'
-#' @author Benjamin Nutter\cr
-#' with thanks to Josh O'Brien and etb (see references)
-#'
-#' @references
-#' See the REDCap API documentation at your institution's REDCap documentation.
-#'
-#' @seealso \code{\link{validateImport}}
-#'
 #' @export
 
 importRecords <- function(rcon, 
@@ -125,6 +128,8 @@ importRecords.redcapApiConnection <- function(rcon,
                                               error_handling = getOption("redcap_error_handling"), 
                                               config = list(), 
                                               api_param = list()){
+  message("importRecords will change how it validates data in version 3.0.0.\n",
+          "We recommend preparing your data for import using castForImport.")
   
    ##################################################################
   # Argument Validation
@@ -279,7 +284,7 @@ importRecords.redcapApiConnection <- function(rcon,
   #** Format the data for REDCap import
   #** Thanks go to:
   #**   https://github.com/etb/my-R-code/blob/master/R-pull-and-push-from-and-to-REDCap.R
-  #**   http://stackoverflow.com/questions/12393004/parsing-back-to-messy-api-strcuture/12435389#12435389
+  #**   https://stackoverflow.com/questions/12393004/parsing-back-to-messy-api-strcuture/12435389#12435389
   
   if (batch.size > 0)
   {
@@ -428,7 +433,7 @@ import_records_unbatched <- function(rcon,
   
   if (response$status_code == "200"){
     if (returnContent %in% c("ids", "auto_ids")){
-      read.csv(text = as.character(response), 
+      utils::read.csv(text = as.character(response), 
                na.strings = "", 
                stringsAsFactors = FALSE)
     } else {
@@ -436,7 +441,7 @@ import_records_unbatched <- function(rcon,
     }
   }
   else 
-    redcap_error(response, error_handling = "error")
+    redcapError(response, error_handling = "error")
 }
 
 #####################################################################
